﻿

'/**
'--------------------------------------------------
' Version .....: 2014/01/31
'               [dd/mm/aaaa] 
' Original en .: Util
' Patrón mvvm 
'--------------------------------------------------
'
'
' Clase .....: NotificationObject
'              InvalidPropertyNameException
'
' Lenguaje ..: Visual Basic .NET (2012)
' Autor .....: Joaquin (jms32) Medina Serrano; en {AGUILA-joaquin}
' Emilio ....: jms32@eresmas.net
'
' Descripción..:
'  - Clase que encapsula la interfaz [INotifyPropertyChanged]

' Bibliografia
' <CallerMemberName> 
' Se puede utilizar el atributo CallerMemberName para evitar especificar 
'  el nombre de miembro como argumento String para el método denominado.  
'  Mediante esta técnica, se evita el problema de que Refactorización de  
'  cambio de nombre no cambie los valores String.  
'  Esta ventaja es especialmente útil para las siguientes tareas: 
' * Usar seguimiento y rutinas de diagnóstico. 
' * Implementar la interfaz INotifyPropertyChanged al enlazar datos.  
'   Esta interfaz permite que la propiedad de un objeto notifique a un  
'   control enlazado que la propiedad ha cambiado, de forma que se pueda 
'   mostrar información actualizada. Sin el atributo CallerMemberName, 
'   se debe especificar el nombre de propiedad como un literal. 
'
'  <ConditionalAttribute("DEBUG")> _
'  El procedimiento que haya sido marcado con el atributo Conditional  
'  se incluirá siempre en la aplicación compilada; sin embargo,  
'  las llamadas al procedimiento sólo se incluirán si se ha definido  
'  la constante de compilación y si su valor es distinto de cero.  
'  En caso contrario, se descartarán estas llamadas.  
'  Esta práctica produce el código más eficaz sin forzarle a agregar  
'  demasiadas directivas en su listado.

'  <DebuggerStepThroughAttribute>
' significa que cando se ejecuta paso a paso pulsando la tecla 
' [F8], paso a paso por instrucciones este codigo se comporta 
' como [Mayus + F8] Paso a paso por procedimientos]
' es decir no se puede establecer un punto de interrupcion 
' en el interior de la funcion
',    DebuggerStepThroughAttribute

'  ' ---------
' ' Información del llamador (C# y Visual Basic)
' ' http://msdn.microsoft.com/es-es/library/hh534540.aspx
'
' ' INotifyPropertyChanged (Interfaz)
' ' http://msdn.microsoft.com/es-es/library/system.componentmodel.inotifypropertychanged.aspx
' 
' ---------------------------------------------- 
' Historial de revisiones (Formato de fecha DD/MM/AAAA 00:00:00.)
' - 2013/01/30 - Creación de la clase 
' - 2013/03/06 - Añado Idisposable
' - 2013/04/01 - Añado INotifyPropertyChanging
' - 2013/09/15 - Simplifico la clase. Elimino la propiedad [nombre porpiedad cambiado]
' -            - Elimino la herencia [Freezable]
' - 2014/01/29 - Cambio la documentacion de la clase
'
' -------------------------------------------------
'*/
'


'----------------------------------------------------------
Option Explicit On
Option Strict On
Option Infer Off
Option Compare Binary




''' <summary>
'''  Clase que encapsula la interfaz [INotifyPropertyChanged]
''' </summary>
''' <remarks></remarks>
<Serializable()> _
Public MustInherit Class ImplementacionINotifyPropertyChanged
    Implements System.ComponentModel.INotifyPropertyChanged,  _
               System.ComponentModel.INotifyPropertyChanging,  _
               System.IDisposable


#Region "Ejemplo de implementacion de una propiedad usando esta clase"


    '' --------------------------------------------------------
    '' Ejemplo de una propiedad implementando los eventos
    ''     [OnPropertyChanging] y [PropertyChanged]
    '' --------------------------------------------------------
    'Private mApellidos As String


    'Public Property Apellidos() As String
    '    Get
    '        Return mApellidos
    '    End Get
    '    Set(ByVal value As String)
    '        If mApellidos <> value Then
    '            '' --------------------------------------------------------
    '            '' Disparar el evento [Changing] (Usando la función [OnPropertyChanged]) 
    '            ' Call OnPropertyChanging()
    '            '' --------------------------------------------------------
    '            mApellidos = value
    '            ' --------------------------------------------------------
    '            ' Disparar el evento [Changed] (Usando la función [OnPropertyChanged]) 
    '            Call OnPropertyChanged()
    '        End If
    '    End Set
    'End Property



    ''-----------------------------------------------
    '' con todos los comentarios a modo de ayuda
    ''-----------------------------------------------
    'Public Property Apellidos() As String
    '    Get
    '        Return mApellidos
    '    End Get
    '    Set(ByVal value As String)
    '        If mApellidos <> value Then
    '            '
    '            ' --------------------------------------------------------
    '            ' Disparar el evento [Changing] (Usando la función [OnPropertyChanged]) 
    '            ' Posibles alternativas (mejor la primera o la tercera porque evitan errores de mecanografia)
    '            Call OnPropertyChanging()
    '            Call OnPropertyChanging("Apellidos")
    '            Call OnPropertyChanging(System.Reflection.MethodBase.GetCurrentMethod.Name)
    '            '
    '            '' --------------------------------------------------------
    '            mApellidos = value
    '            ' --------------------------------------------------------
    '            ' Disparar el evento [Changed] (Usando la función [OnPropertyChanged]) 
    '            ' ------------------------------
    '            ' ¡¡ Observación !!
    '            ' Si obtengo el nombre de la propiedad que cambia mediante reflexión, 
    '            ' Por ejemplo con: [System.Reflection.MethodBase.GetCurrentMethod.Name]
    '            ' el nombre de la propiedad que se obtiene esta precedido
    '            ' por el prefijo [get_], o [set_]. Esta contemplado y tratado en el 
    '            ' codigo de la funcion [OnPropertyChanged]
    '            ' --------------------------------------------------------
    '            ' Posibles alternativas (mejor la primera o la tercera porque evitan errores de mecanografia)
    '            Call OnPropertyChanged()
    '            Call OnPropertyChanged("Apellidos")
    '            Call OnPropertyChanged(System.Reflection.MethodBase.GetCurrentMethod.Name)
    '        End If
    '    End Set
    'End Property
    ''-----------------------------------------------------------------------



    ''---------------------------------------------------------------------------
    '' Con una propiedad de dependencia
    ''---------------------------------------------------------------------------
    ''
    ' ''' <summary>
    ' '''  Propiedad de dependencia [Nombre]
    ' ''' </summary>
    'Public Shared ReadOnly NombreProperty As DependencyProperty = _
    '              DependencyProperty.Register("Nombre", _
    '                       GetType(String), _
    '                       GetType(Persona), _
    '                       New PropertyMetadata(defaultValue:=String.Empty))
    ''
    ' ''' <summary>
    ' '''  El nombre de la persona
    ' ''' </summary>
    'Public Property Nombre As String
    '    Get
    '        Return Convert.ToString(GetValue(NombreProperty))
    '    End Get
    ''
    '    Set(ByVal value As String)
    '        Dim oldValue As String = Convert.ToString(GetValue(NombreProperty))
    ''
    '        If oldValue <> value Then
    '            Call OnPropertyChanging()
    '            Call OnPropertyChanging("Nombre")
    '            Call OnPropertyChanging(System.Reflection.MethodBase.GetCurrentMethod.Name)
    '            '
    '            SetValue(NombreProperty, value)
    '            '
    '            Call OnPropertyChanged()
    '            Call OnPropertyChanged("Nombre")
    '            Call OnPropertyChanged(System.Reflection.MethodBase.GetCurrentMethod.Name)
    '        End If
    '    End Set
    'End Property
    '
    '



#End Region


#Region "Variables de la clase"

    ''' <summary>
    '''  Variable de clase para el nombre de la propiedad que ha cambiado
    ''' </summary>
    Private _notifyNombrePropiedadQueCambia As String = String.Empty

    ''' <summary>
    '''  El nombre de la propiedad que ha cambiado
    ''' </summary>
    Public ReadOnly Property NotifyNombrePropiedadQueCambia As String
        ' Se carga en la función [OnPropertyChanged]
        Get
            Return _notifyNombrePropiedadQueCambia
        End Get
    End Property

#End Region


    Protected Sub New()
        ' no hacer nada
    End Sub

#Region "Implements PropertyChanged"

    '-----------------------------------------------------------------------
    ' !!!Observación!!! No se usa un evento genérico EventHandler
    ' Porque se produce un error al serializar eventos Genéricos
    ''' <summary>
    '''  Evento [PropertyChanged] de la interfaz [INotifyPropertyChanged]
    ''' </summary>
    <NonSerializedAttribute()> _
    Public Event PropertyChanged As  _
                System.ComponentModel.PropertyChangedEventHandler _
                Implements System.ComponentModel.INotifyPropertyChanged.PropertyChanged

    '----------------------------------------------------------------------
    ''' <summary>Función que dispara el evento [PropertyChanged]</summary>
    ''' <param name="e">
    '''  Se espera un objeto del tipo [ComponentModel.PropertyChangedEventArgs]
    '''  Esta funcion se usa pra relanzar un evento
    ''' </param>
    Protected Overloads Sub OnPropertyChanged(ByVal e As ComponentModel.PropertyChangedEventArgs)
        If e Is Nothing Then
            Throw New ArgumentException("No se admiten valores Nothing")
        End If
        Call OnPropertyChanged(e.PropertyName)
    End Sub



    '----------------------------------------------------------------------
    ''' <summary>Función que dispara el evento [PropertyChanged]</summary>
    ''' <param name="nombreDeLaPropiedadChanged">
    '''  <para>Se espera una cadena de texto con uno de los siguientes valores:</para>
    '''  <para> a) Una cadena vacía, en este caso la función averiguara el 
    '''            nombre de la propiedad a través de su 
    '''            atributo [CallerMemberName]]</para>
    '''  <para> b) Una cadena de texto con el nombre de 
    '''             la propiedad (Ejemplo, "Apellidos") </para>
    '''  <para> c) Una cadena proporcionada por reflexión
    '''            [System.Reflection.MethodBase.GetCurrentMethod.Name]</para>
    ''' </param>
    ''' <remarks>
    ''' <code>
    '''     system.componentmodel.inotifypropertychanged(VS.95).aspx
    '''          http://msdn.microsoft.com/es-es/library/
    '''     Bibliografia [CallerMemberName>]
    '''          http://msdn.microsoft.com/es-es/library/hh534540.aspx
    ''' </code>
    '''</remarks>
    ''' <exception cref="InvalidPropertyNameException">
    '''    <para> Si el nombre de la propiedad no existe en la clase</para>
    '''    <para> o bien si</para>
    '''    <para> Hay un error de lógica. No se puede lanzar un evento [Changed] 
    '''           desde un constructor, desde un constructor estático,
    '''            o desde un destructor</para>
    ''' </exception>
    Protected Overloads Sub OnPropertyChanged(
        <System.Runtime.CompilerServices.CallerMemberNameAttribute>
                   Optional ByVal nombreDeLaPropiedadChanged As String = Nothing)
        ' ----------------------------------
        ' Evitar problemas tontos
        If String.IsNullOrWhiteSpace(nombreDeLaPropiedadChanged) = False Then
            ' ----------------------------------
            ' Quitar los  prefijos [get_], o [set_]
            nombreDeLaPropiedadChanged = CorregirNombrePropiedad(nombreDeLaPropiedadChanged)
            ' ----------------------------------
            ' Vuelvo a preguntar porque despues de las correcciones
            ' puede venir una cadena vacia y asi evito un error
            If String.IsNullOrWhiteSpace(nombreDeLaPropiedadChanged) = False Then
                ' ----------------------------------
                ' Comprobar que el nombre de la propiedad existe en la clase
                Call VerificarExistenciaNombrePropiedad(Me, nombreDeLaPropiedadChanged)
                ' ----------------------------------
                ' actualizar el nombre de la propiedad que cambia
                _notifyNombrePropiedadQueCambia = nombreDeLaPropiedadChanged
                ' Disparar el evento
                RaiseEvent PropertyChanged( _
                          Me,
                          New System.ComponentModel.PropertyChangedEventArgs(nombreDeLaPropiedadChanged))
            End If
        End If
    End Sub


#End Region

#Region "Funciones Auxiliares"

    ''' <summary>
    '''    Quita el prefijo [Get_] y/o [Set_] del nombre de la propiedad si existe
    ''' </summary>
    ''' <param name="nombreDeLaPropiedadChanged">El nombre de la porpiedad que cambia</param>
    ''' <returns>La propiedad corregida</returns>
    ''' <remarks>
    '''    Si obtengo el nombre de la propiedad que cambia mediante reflexión, 
    '''    Por ejemplo con: [System.Reflection.MethodBase.GetCurrentMethod.Name]
    '''    el nombre de la propiedad que se obtiene esta precedido
    '''    por el prefijo [get_], o [set_]
    '''</remarks>
    ''' <exception cref="InvalidPropertyNameException">
    '''  Hay un error de de logica. No se puede lanzar un evento [Changed] 
    '''  desde un constructor, desde un constructor estatico, o desde un desctructor
    '''</exception>
    Private Shared Function CorregirNombrePropiedad(ByVal nombreDeLaPropiedadChanged As String) As String

        ' El valor de retorno (por defecto)
        ' por si es null, o cadenas de espacios
        Dim auxCadena As String = String.Empty

        If String.IsNullOrWhiteSpace(nombreDeLaPropiedadChanged) = False Then
            ' este es el valor recibido que se devuelve si esta correcto
            auxCadena = nombreDeLaPropiedadChanged

            ' buscar [Get] y/o [Set]
            Dim nombreMayus As String =
                nombreDeLaPropiedadChanged.ToUpper(
                         System.Globalization.CultureInfo.CurrentCulture)

            ' Corregir los nombres de miembro que se devuelven cuando se utiliza
            ' reflexion para obtener el nombre de la propiedad. 
            ' http://msdn.microsoft.com/es-es/library/hh534540.aspx

            If nombreMayus.StartsWith("SET_", StringComparison.CurrentCultureIgnoreCase) = True OrElse
               nombreMayus.StartsWith("GET_", StringComparison.CurrentCultureIgnoreCase) = True Then
                auxCadena = auxCadena.Remove(0, 4)
            End If

            '---------------------------------------------------------------------------------
            'Observación
            '   hay que declarar un error porque es un error de lógica del programa
            '   no se puede lanzar un evento Changed desde un constructor o un destructor
            '   o un operador definido por el usuario
            '---------------------------------------------------------------------------------
            ' Constructor   La cadena “.ctor” 
            If nombreMayus.StartsWith(".CTOR",
                                      StringComparison.CurrentCultureIgnoreCase) = True Then
                Throw New InvalidPropertyNameException(
                          "Error de lógica. No se puede lanzar " & _
                          "un evento [Changed] desde un constructor")
            End If
            'Constructor estático  La cadena “.cctor” 
            If nombreMayus.StartsWith(".CCTOR",
                                      StringComparison.CurrentCultureIgnoreCase) = True Then
                Throw New InvalidPropertyNameException(
                          "Error de lógica. No se puede lanzar  " & _
                          "un evento [Changed] desde un constructor estático")
            End If
            'Un destructor  La cadena “Finalize” 
            If nombreMayus.StartsWith("FINALIZE",
                                      StringComparison.CurrentCultureIgnoreCase) = True Then
                Throw New InvalidPropertyNameException(
                          "Error de lógica. No se puede lanzar  " & _
                          "un evento [Changed] desde un destructor")
            End If

            If nombreMayus.StartsWith("OP_",
                          StringComparison.CurrentCultureIgnoreCase) = True Then
                Throw New InvalidPropertyNameException(
                          "Error de lógica. No se puede lanzar  " & _
                          "un evento [Changed] desde operador definido por el usuario")
            End If

        End If
        Return auxCadena
    End Function


    ''' <summary>
    '''  Verifica que el nombre de la propiedad exista en la clase
    '''  La propiedad debe estar calificada como [Public].
    ''' </summary>
    ''' <param name="laClaseDerivada">El objeto del que se listan las propiedades</param>
    ''' <param name="nombreDeLaPropiedadChanged">El nombre de la propiedad que cambia</param>
    ''' <remarks>
    ''' <para>El atributo [ConditionalAttribute("DEBUG")] 
    '''       El procedimiento que haya sido marcado con el atributo Conditional  
    '''       se incluirá siempre en la aplicación compilada; sin embargo,  
    '''       las llamadas al procedimiento sólo se incluirán si se ha definido  
    '''       la constante de compilación y si su valor es distinto de cero.  
    '''       En caso contrario, se descartarán estas llamadas.
    ''' </para> 
    ''' <para>El atributo [DebuggerStepThrough]  
    '''       significa que cuando se ejecuta paso a paso pulsando la tecla 
    '''       [F8], paso a paso por instrucciones este código se comporta 
    '''       como [Mayus + F8] Paso a paso por procedimientos]
    ''' </para>
    ''' <code> 
    '''   Código original en:
    '''       WPF Apps With The Model-View-ViewModel Design Pattern
    '''       http://msdn.microsoft.com/es-es/magazine/dd419663.aspx
    ''' </code>
    '''</remarks>
    ''' <exception cref="InvalidPropertyNameException">
    '''           Si el nombre de la propiedad no existe en la clase 
    ''' </exception>
    <ConditionalAttribute("DEBUG"),
     DebuggerStepThrough()> _
    Private Shared Sub VerificarExistenciaNombrePropiedad(
                        ByVal laClaseDerivada As Object,
                        ByVal nombreDeLaPropiedadChanged As String)

        '--------------------------------------------------------
        'Colección con las propiedades de la clase derivada
        Dim properties As System.ComponentModel.PropertyDescriptorCollection =
                          System.ComponentModel.TypeDescriptor.GetProperties(laClaseDerivada)
        '--------------------------------------------------------
        ' Evitar problemas con valores Null
        If Not (properties Is Nothing) Then
            ' Comprobar si existe la propiedad
            If (properties(nombreDeLaPropiedadChanged) Is Nothing) Then
                '--------------------------------
                ' La propiedad no existe
                ' Disparar el error personalizado de propiedad no existe 
                ' en la clase derivada y montar un mensaje explicativo extenso
                '--------------------------------
                ' el nombre de la clase base es el nombre de esta clase 
                ' [ImplementacionINotifyPropertyChanged]
                Dim elNombreDeLaClaseBase As String =
                    System.Reflection.MethodBase.GetCurrentMethod.DeclaringType.Name
                '--------------------------------
                Dim mensajeError As String
                mensajeError = InvalidPropertyNameException.MontarElInformeDelError(
                    elNombreDeLaClaseBase,
                    laClaseDerivada,
                    nombreDeLaPropiedadChanged)
                '----------
                Throw New InvalidPropertyNameException(mensajeError)
            End If
            properties = Nothing
        End If
    End Sub


#End Region

#Region "Implementts PropertyChanging"

    <NonSerializedAttribute()> _
    Public Event PropertyChanging As  _
           System.ComponentModel.PropertyChangingEventHandler _
           Implements System.ComponentModel.INotifyPropertyChanging.PropertyChanging



    '----------------------------------------------------------------------
    ''' <summary>Función que dispara el evento [PropertyChanging]</summary>
    ''' <param name="e">
    '''  Se espera un objeto del tipo [ComponentModel.PropertyChangingEventArgs]
    '''  Esta funcion se usa para relanzar un evento
    ''' </param>
    Protected Overloads Sub OnPropertyChanging(ByVal e As ComponentModel.PropertyChangingEventArgs)
        If e Is Nothing Then
            Throw New ArgumentException("No se admiten valores Nothing")
        End If
        Call OnPropertyChanging(e.PropertyName)
    End Sub



    '----------------------------------------------------------------------
    ''' <summary>Función que dispara el evento [PropertyChanging]</summary>
    ''' <param name="nombreDeLaPropiedadChanging">
    '''    Una cadena con el nombre de la propiedad que va ha cambiar
    '''    Se espera un valor del tipo [System.Reflection.MethodBase.GetCurrentMethod.Name]
    ''' </param>
    Protected Overloads Sub OnPropertyChanging(
        <System.Runtime.CompilerServices.CallerMemberName> Optional nombreDeLaPropiedadChanging As String = Nothing)
        ' ----------------------------------
        ' Evitar problemas tontos
        If String.IsNullOrWhiteSpace(nombreDeLaPropiedadChanging) = False Then
            ' ----------------------------------
            ' ----------------------------------
            ' Quitar los  prefijos [get_], o [set_]
            nombreDeLaPropiedadChanging = CorregirNombrePropiedad(nombreDeLaPropiedadChanging)
            ' ----------------------------------
            ' Vuelvo a preguntar porque despues de las correcciones
            ' puede venir una cadena vacia y asi evito un error
            If String.IsNullOrWhiteSpace(nombreDeLaPropiedadChanging) = False Then
                '--------------------------
                ' Comprobar que el nombre de la propiedad existe en la clase
                Call VerificarExistenciaNombrePropiedad(Me, nombreDeLaPropiedadChanging)
                ' ----------------------------------
                ' actualizar el nombre de la propiedad que cambia
                _notifyNombrePropiedadQueCambia = nombreDeLaPropiedadChanging
                ' Disparar el evento
                RaiseEvent PropertyChanging( _
                          Me,
                          New System.ComponentModel.PropertyChangingEventArgs(nombreDeLaPropiedadChanging))
            End If
        End If
    End Sub

#End Region

#Region "IDisposable Support"
    Private _disposedValue As Boolean ' Para detectar llamadas redundantes

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not Me._disposedValue Then
            If disposing Then
                '  eliminar estado administrado (objetos administrados).
            End If
            Me.NotifyNombrePropiedadQueCambia = Nothing
        End If
        Me._disposedValue = True
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

#End Region

End Class


''' <summary>
'''  Error especifico para indicar que el nombre de una propiedad 
'''  pasado en [PropertyChanged] es erroneo y no existe en la clase
''' </summary>
<Serializable()> _
Public Class InvalidPropertyNameException
    Inherits System.Exception


    ''' <summary>
    ''' Inicializa una nueva instancia con un mensaje de error
    ''' </summary>
    ''' <param name="message">El mensaje que describe el error.</param>
    Protected Friend Sub New(ByVal message As String)
        MyBase.New(message)
        If String.IsNullOrWhiteSpace(message) = True Then
            Throw New ArgumentNullException("message")
        End If
    End Sub


    ''' <summary>
    '''  Genera un informe para usarlo como mensaje del error personalizado
    ''' </summary>
    ''' <param name="nombreClaseBase">debe ser [ImplementacionINotifyPropertyChanged]</param>
    ''' <param name="laClaseDerivada">El objeto del que se listan las propiedades</param>
    ''' <param name="nombreDeLaPropiedadChanged">El nombre de la porpiedad que cambia</param>
    ''' <remarks></remarks>
    Public Shared Function MontarElInformeDelError(ByVal nombreClaseBase As String,
                                                   ByVal laClaseDerivada As Object,
                                                   ByVal nombreDeLaPropiedadChanged As String) As String

        Dim cultura As System.Globalization.CultureInfo
        cultura = System.Globalization.CultureInfo.CurrentCulture

        If String.IsNullOrWhiteSpace(nombreClaseBase) = True Then
            Using sw As New System.IO.StringWriter(cultura)
                sw.WriteLine("Problemas en la generacion del informe de error")
                sw.WriteLine("El [nombreClaseBase] es una cadena [Nothing],[Empty], o [WhiteSpace]")
                sw.Flush()
                Return sw.ToString
            End Using
        End If

        If laClaseDerivada Is Nothing Then
            Using sw As New System.IO.StringWriter(cultura)
                sw.WriteLine("Problemas en la generacion del informe de error")
                sw.WriteLine("El [laClaseDerivada] tiene un valor [Nothing]")
                sw.Flush()
                Return sw.ToString
            End Using
        End If

        If String.IsNullOrWhiteSpace(nombreDeLaPropiedadChanged) = True Then
            Using sw As New System.IO.StringWriter(cultura)
                sw.WriteLine("Problemas en la generacion del informe de error")
                sw.WriteLine("El [nombreDeLaPropiedadChanged] es una cadena [Nothing],[Empty], o [WhiteSpace]")
                sw.Flush()
                Return sw.ToString
            End Using
        End If


        Dim listaPropiedades As List(Of String)
        Dim nombreClaseDerivadaCadena As String
        Try
            ' La lista de propiedades de la clase
            listaPropiedades = ListaPropiedadesDeLaClase(laClaseDerivada)
            ' el nombre de la case derivada como cadena
            nombreClaseDerivadaCadena = System.ComponentModel.TypeDescriptor.GetClassName(laClaseDerivada)
        Catch ex As Exception
            Using sw As New System.IO.StringWriter(cultura)
                sw.WriteLine("Problemas en la generacion del informe de error")
                sw.WriteLine(ex.Message)
                sw.Flush()
                If ex.InnerException Is Nothing Then
                    Throw New Exception(sw.ToString, ex)
                Else
                    Throw New Exception(sw.ToString, ex.InnerException)
                End If
            End Using
        End Try

        '-------------------------------------------------
        ' llamar al generador de infomes y devolverlo
        Return MontarElInformeDelError(listaPropiedades, nombreClaseBase, nombreClaseDerivadaCadena, nombreDeLaPropiedadChanged)
    End Function

    '-------------------------------------------------------------
    ' Función auxiliar
    ''' <summary>
    '''  Genera un informe para usarlo como mensaje del error personalizado
    ''' </summary>
    ''' <param name="listaPropiedades"> [List(Of String)] La lista de propiedades de la clase</param>
    ''' <param name="nombreClaseBase"> El nombre de la clase base </param>
    ''' <param name="nombreClaseDerivada">El nombre de la clase derivada</param>
    ''' <param name="nombreDeLaPropiedadChanged">El nombre de la propiedad Changed </param>
    ''' <returns>un informe para usarlo como mensaje del error personalizado</returns>
    Private Shared Function MontarElInformeDelError(
                    ByVal listaPropiedades As List(Of String),
                    ByVal nombreClaseBase As String,
                    ByVal nombreClaseDerivada As String,
                    ByVal nombreDeLaPropiedadChanged As String) As String


        '-------------------------------------------------------
        'Esta funcion genera el siguiente informe
        '-------------------------------------------------------

        '
        '
        '/Start Informe de error
        '----------------------------------
        'ERROR EN: WpfCifraSimetrica2104GUI.Colega
        'FECHA (UTC)  : 31/01/2014 11:54:32 
        'DESCRIPCION: La propiedad [NombreMalo] no existe en la clase [WpfCifraSimetrica2104GUI.Colega]
        '----------------------------------
        '
        'El error detectado es el siguiente: 
        'La clase [WpfCifraSimetrica2104GUI.Colega] que hereda de la clase base [ImplementacionINotifyPropertyChanged]  
        'ha declarado que se ha modificado el valor de la propiedad [NombreMalo]  
        'y esta propiedad no existe en la clase derivada [WpfCifraSimetrica2104GUI.Colega] 
        '
        '----------------------------------
        'A modo de ayuda:
        'a) La clase [ImplementacionINotifyPropertyChanged] es una clase
        '   situada en el ensamblado [MVVF2013Dll.dll]
        '   que implementa la interfaz [INotifyPropertyChanged]
        '   y contiene la funcionalidad necesaria para disparar el evento [PropertyChanged]
        '   Se usa como clase base por la clase derivada [WpfCifraSimetrica2104GUI.Colega] 
        '
        'b) Lista de propiedades de la clase [WpfCifraSimetrica2104GUI.Colega] (derivada)
        '   - Nombre 
        '   - NotifyNombrePropiedadQueCambia 
        '/Eof Lista
        '----------------------------------
        '/Eof Informe de error
        '
        '




        Dim cultura As System.Globalization.CultureInfo
        cultura = System.Globalization.CultureInfo.CurrentCulture

        ' el ensamblado donde esta la clase base
        ' es decir la clase [ImplementacionINotifyPropertyChanged]
        Dim ensambladoClaseBase As String = System.Reflection.MethodBase.GetCurrentMethod.Module.Name

        Using sw As New System.IO.StringWriter(cultura)

            sw.WriteLine("")
            sw.WriteLine("")
            sw.WriteLine("/Start Informe de error")
            sw.WriteLine("----------------------------------")
            sw.WriteLine("ERROR EN: {0}", nombreClaseDerivada)
            sw.WriteLine("FECHA (UTC)  : {0} ", Date.Now.ToUniversalTime.ToString(cultura))
            sw.WriteLine("DESCRIPCION: La propiedad [{0}] no existe en la clase [{1}]",
                         nombreDeLaPropiedadChanged,
                         nombreClaseDerivada)
            sw.WriteLine("----------------------------------")

            sw.WriteLine("")
            sw.WriteLine("El error detectado es el siguiente: ")
            sw.WriteLine("La clase [{0}] que hereda de la clase base [{1}]  ", nombreClaseDerivada, nombreClaseBase)
            sw.WriteLine("ha declarado que se ha modificado el valor de la propiedad [{0}]  ", nombreDeLaPropiedadChanged)
            sw.WriteLine("y esta propiedad no existe en la clase derivada [{0}] ", nombreClaseDerivada)
            sw.WriteLine("")

            sw.WriteLine("----------------------------------")
            sw.WriteLine("A modo de ayuda:")

            sw.WriteLine("a) La clase [{0}] es una clase", nombreClaseBase)
            sw.WriteLine("   situada en el ensamblado [{0}]", ensambladoClaseBase)
            sw.WriteLine("   que implementa la interfaz [INotifyPropertyChanged]")
            sw.WriteLine("   y contiene la funcionalidad necesaria para disparar el evento [PropertyChanged]")
            sw.WriteLine("   Se usa como clase base por la clase derivada [{0}] ", nombreClaseDerivada)

            sw.WriteLine("")
            sw.WriteLine("b) Lista de propiedades de la clase [{0}] (derivada)", nombreClaseDerivada)
            If listaPropiedades.Count < 1 Then
                sw.WriteLine("   - {0} ", " ¡¡ATENCION !! - Esta clase no tiene propiedades ")
            Else
                Dim unaPropiedad As String
                For Each unaPropiedad In listaPropiedades
                    sw.WriteLine("   - {0} ", unaPropiedad)
                Next unaPropiedad
            End If
            sw.WriteLine("/Eof Lista")
            sw.WriteLine("----------------------------------")
            sw.WriteLine("/Eof Informe de error")
            sw.WriteLine("")
            sw.WriteLine("")
            sw.Flush()
            Return sw.ToString
        End Using

    End Function

    '-------------------------------------------------------------
    ' Función auxiliar 
    ''' <summary>
    '''   Devuelve una lista de cadenas de texto [List(Of String)]
    '''   que contiene los nombres de las propiedades de la clase
    ''' </summary>
    ''' <param name="unaClase">El objeto del que se listan las propiedades</param>
    ''' <returns>Un objeto [List(Of String)] que contiene los nombres de las propiedades de la clase</returns>
    ''' <remarks></remarks>
    Private Shared Function ListaPropiedadesDeLaClase(ByVal unaClase As Object) As List(Of String)
        '--------------------------------------------------------
        Dim listaPropiedades As New List(Of String)
        '--------------------------------------------------------
        ' Evitar problemas con valores [Nothing]
        If Not (unaClase Is Nothing) Then

            'colección de las propiedades de la clase 
            Dim properties As System.ComponentModel.PropertyDescriptorCollection =
                System.ComponentModel.TypeDescriptor.GetProperties(unaClase)
            ' 
            If Not (properties Is Nothing) Then
                For Each unaPropiedad As System.ComponentModel.PropertyDescriptor In properties
                    listaPropiedades.Add(unaPropiedad.Name)
                Next
            End If
        End If
        Return listaPropiedades
    End Function

End Class
